#include "TCPServerSocket.h"
#include "EPoll.h"
#include "Session.h"
#include "Exception.h"
#include "Log.h"

namespace core {

   TCPServerSocket::TCPServerSocket(EPoll &ePoll, Service &service, IPAddress address) : TCPSocket(ePoll), service(service) {    
      setDescriptor(socket(AF_INET, SOCK_STREAM, 0));
      int yes = 1;
      setsockopt(getDescriptor(), SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int));
      if(bind(getDescriptor(), address.getPointer(), address.addressLength) < 0) 
			throw coreutils::Exception("Error on bind to socket: " + std::to_string(errno));
      if(listen(getDescriptor(), 10) < 0) 
			throw coreutils::Exception("Error on listen to socket");
      ePoll.registerSocket(this);   
   }

   TCPServerSocket::~TCPServerSocket() {
      close(getDescriptor());
   }
      
   void TCPServerSocket::onDataReceived(std::string data) {
      Session *session = accept();
      if(session) service.sessions.push_back(session);   
   }
   
   Session * TCPServerSocket::accept() {      
      Session *session = service.getSocketAccept(ePoll);      
      session->setDescriptor(::accept(getDescriptor(), (struct sockaddr *)&session->ipAddress.addr, &session->ipAddress.addressLength));

//      if(blackList && blackList->contains(session->ipAddress.getClientAddress())) {
//         session->shutdown();
//         Log(LOG_WARN) << "Client at IP address " << session->ipAddress.getClientAddress() << " is blacklisted and was denied a connection.";
//         return NULL;
//      }
//
//      if(whiteList && !whiteList->contains(session->ipAddress.getClientAddress())) {
//         session->shutdown();
//         Log(LOG_WARN) << "Client at IP address " << session->ipAddress.getClientAddress() << " is not authorized and was denied a connection.";
//         return NULL;
//      }
      
      ePoll.registerSocket(session);   
      coreutils::Log(coreutils::LOG_DEBUG_2) << "Session started on socket " << session->getDescriptor() << ".";
      return session;
   }
   
   int TCPServerSocket::processCommand(std::string command, Session *session, std::stringstream &data) {      
      int sequence = 0;      
      for(auto *sessionx : service.sessions) {
         data << "|" << ++sequence;
         sessionx->output(data); 
         data << "|" << std::endl;
      }
   }
   
}
